import nose.core

from django_nose.plugin import ResultPlugin
from django_nose.runner import NoseTestSuiteRunner, _get_plugins_from_settings


class TestSuiteRunner(NoseTestSuiteRunner):
    """
    NoseRunner which works with tddspry.

    Basically the difference is that we remove DjangoSetUpPlugin as tddspry
    does setup itself and add ``--with-django`` option.
    """
    def run_suite(self, nose_argv):
        # Enable tddspry
        nose_argv.append('--with-django')

        result_plugin = ResultPlugin()
        plugins_to_add = [result_plugin]

        for plugin in _get_plugins_from_settings():
            plugins_to_add.append(plugin)

        nose.core.TestProgram(argv=nose_argv,
                              exit=False,
                              addplugins=plugins_to_add)

        return result_plugin.result



"""
``DatabaseTestCase`` and ``HttpTestCase`` can call additional helpers by
``helper(name, *args, **kwargs)`` method. Now ``tddspry.django.helpers``
provides:

.. autofunction :: create_profile
.. autofunction :: create_staff
.. autofunction :: create_superuser
.. autofunction :: create_user
.. autofunction :: registration

Usage
-----

For create new non-active user in test case use next snippet::

    from tddspry.django import DatabaseTestCase


    class TestNonActiveUser(DatabaseTestCase):

        def test_active(self):
            user = self.helper('create_user', active=False)
            self.assert_false(user.is_active)

Also you can to access any additional helper function or var directly by
``helpers`` attribute. For example, to access default values for ``username``,
``password`` and ``email`` user fields use next snippet::

    from tddspry.django import DatabaseTestCase


    class TestHelpersVars(DatabaseTestCase):

        def test_default_values(self):
            user = self.helper('create_user', raw=True)
            self.assert_equal(user.username, self.helpers.USERNAME)
            self.assert_equal(user.password, self.helpers.PASSWORD)
            self.assert_equal(user.email, self.helpers.EMAIL)

**Note:** You don't need to send ``obj`` argument to additional helper via
``helper`` method. But if you want to call additional helper directly you must
send ``DatabaseTestCase`` or ``HttpTestCase`` object as first argument.

"""

from tddspry.django.helpers.auth import *
from tddspry.django.helpers.registration import *



"""
``tddspry`` provides ``NoseTestCase`` class to base functional testing.

.. autoclass :: tddspry.NoseTestCase
   :members:

"""

from tddspry.cases import *


VERSION = (0, 4, 'beta')


def get_version():
    """
    Returns human-readable version of your **tddspry** installation.
    """
    intjoin = lambda data, sep=None: (sep or '.').join(map(str, data))

    if VERSION[-1] is not None:
        if isinstance(VERSION[-1], int):
            version = intjoin(VERSION)
        else:
            version = '%s-%s' % (intjoin(VERSION[:-1]), VERSION[-1])
    else:
        version = intjoin(VERSION[:-1])

    return version



from django.contrib.auth.models import User


__all__ = ('EMAIL', 'PASSWORD', 'USERNAME', 'create_profile', 'create_staff',
           'create_superuser', 'create_user')


EMAIL = 'test_email@domain.com'
PASSWORD = 'test_password'
USERNAME = 'test_username'


def create_profile(obj, user, klass, **kwargs):
    """
    Create profile for given user.
    """
    kwargs.update({'user': user})
    return klass.objects.create(**kwargs)


def create_staff(obj, username=None, password=None, email=None, raw=False):
    """
    Create Django user with staff rights.
    """
    return create_user(obj, username, password, email, staff=True, raw=raw)


def create_superuser(obj, username=None, password=None, email=None, raw=False):
    """
    Create Django user with superuser and staff rights.
    """
    return create_user(obj, username, password, email, staff=True,
                       superuser=True, raw=raw)


def create_user(obj, username=None, password=None, email=None, active=True,
                staff=False, superuser=False, raw=False):
    """
    Create Django user with given names or with default if absent.
    """
    user = User(username=username or USERNAME,
                email=email or EMAIL)

    user.is_active = active
    user.is_staff = staff
    user.is_superuser = superuser

    if raw:
        user.password = password or PASSWORD
    else:
        user.set_password(password or PASSWORD)

    user.save()
    return user



"""
Plugin that make able to test Django projects or applications with nose
library.
"""

import logging
import os
import re
import sys

from nose.plugins.base import Plugin
from nose.util import ispackage, resolve_name, skip_pattern


__all__ = ('DjangoPlugin', )


log = logging.getLogger(__name__)
skip_pattern_re = re.compile(skip_pattern)


class DjangoPlugin(Plugin):
    """
    Run nosetests for Django projects or apps. You need to specify settings of
    your project or plugin tries to auto-load from current or child
    directories.
    """
    error_dir = None
    name = 'django'
    settings = None
    verbosity = 1

    def begin(self):
        from django.conf import settings
        from django.core.handlers.wsgi import WSGIHandler
        from django.core.servers.basehttp import AdminMediaHandler
        from django.test.simple import TEST_MODULE

        from tddspry.django.settings import IP, PORT

        from twill import add_wsgi_intercept

        log.debug('DjangoPlugin start')

        # Find to Django models in tests modules for each of ``INSTALLED_APPS``
        for label in settings.INSTALLED_APPS:
            tests = label + '.' + TEST_MODULE

            try:
                self.load_tests(tests)
            except (AttributeError, ImportError):
                pass

        # Setup Django test environment and test database
        self.setup_django()

        # Setup Twill for testing with Django
        app = AdminMediaHandler(WSGIHandler())
        add_wsgi_intercept(IP, PORT, lambda: app)

    def configure(self, options, config):
        Plugin.configure(self, options, config)

        # Do nothing if plugin not enabled
        if not self.enabled:
            return

        # Check that Django and twill libraries available in this system
        if self.enabled:
            for lib in ('Django', 'twill'):
                try:
                    __import__(lib.lower())
                except ImportError, e:
                    log.error('%s not available: %s' % (lib, e))
                    self.enabled = False
                    return

        # Get user defined options
        self.error_dir = options.error_dir
        self.settings = options.settings
        self.verbosity = options.verbosity
        self.test_match_re = config.testMatch

        # Try to load Django project settings
        self.load_settings(self.settings)

        # Make sure that ``TWILL_ERROR_DIR`` set to ``os.environ`` if needed
        if self.error_dir:
            os.environ['TWILL_ERROR_DIR'] = self.error_dir

    def error(self, settings, dirname=None, subdirs=None):
        if settings is not None:
            sys.stderr.write(
                "Error: Can't find the module %r in ``sys.path``.\n(If " \
                "the file settings.py does indeed exist, it's causing an " \
                "ImportError somehow.)\n" % settings)
        else:
            sys.stderr.write(
                "Error: Can't find the file 'settings.py' in the current " \
                "work directory %r and all subdirectories %r. It appears " \
                "you've customized things.\nYou'll have to run `nosetests " \
                "--with-django` passing it your settings module.\n(If the " \
                "file settings.py does indeed exist, it's causing an " \
                "ImportError somehow.)\n" % (dirname, subdirs))
        sys.exit(1)

    def ismodule(self, obj):
        return hasattr(obj, '__file__')

    def ispackage(self, module):
        return module.__file__.rstrip('co').endswith('__init__.py')

    @property
    def legacy_django(self):
        from django import VERSION
        return not (VERSION[0] == 1 and VERSION[1] >= 2)

    def load_settings(self, settings):
        # If settings module was set try to load or die with error
        if settings is not None:
            try:
                resolve_name(settings)
            except (AttributeError, ImportError):
                return self.error(settings)
        else:
            settings = 'settings'

            try:
                resolve_name(settings)
            except (AttributeError, ImportError):
                dirname = os.getcwd()
                loaded = False

                subdirs = \
                    filter(lambda name: os.path.isdir(os.path.join(dirname,
                                                                   name)),
                           os.listdir(dirname))
                subdirs.sort()

                for name in subdirs:
                    settings = name + '.settings'

                    try:
                        resolve_name(settings)
                    except (AttributeError, ImportError, ValueError):
                        pass
                    else:
                        loaded = True
                        break

                if not loaded:
                    self.error(None, dirname, subdirs)

        os.environ['DJANGO_SETTINGS_MODULE'] = settings

    def load_tests(self, basename):
        obj = resolve_name(basename)

        if not self.ismodule(obj) or not self.ispackage(obj):
            return

        dirname = os.path.dirname(obj.__file__)

        childs = os.listdir(dirname)
        childs.sort()
        childs = map(lambda name: (name, os.path.join(dirname, name)), childs)

        for name, fullname in childs:
            if os.path.isdir(fullname) and ispackage(fullname):
                self.load_tests(basename + '.' + name)

            if not os.path.isfile(fullname) or skip_pattern_re.match(name) or \
               not name.endswith('.py') or name == '__init__.py':
                continue

            if self.test_match_re.match(name):
                self.load_tests(basename + '.' + name[:-3])

    def options(self, parser, env=None):
        env = env or os.environ
        Plugin.options(self, parser, env)

        parser.add_option('--django-settings', action='store',
                          default=env.get('DJANGO_SETTINGS_MODULE'),
                          dest='settings', help='Settings module of your ' \
                          'Django project to test. [DJANGO_SETTINGS_MODULE]')
        parser.add_option('--django-verbosity', action='store',
                          default=1, dest='verbosity', type='int',
                          help='Django verbosity level.')
        parser.add_option('--twill-error-dir', action='store',
                          default=env.get('TWILL_ERROR_DIR'), dest='error_dir',
                          help='If `TwillAssertionError` raised store all ' \
                          'output in this directory. [TWILL_ERROR_DIR]')

    def report(self, stream):
        log.debug('DjangoPlugin report')

        # Destroy Django test database and teardown Django test environment
        self.teardown_django()

    def setup_django(self):
        from django.conf import settings

        # If Django < 1.2
        if self.legacy_django:
            from django.db import connection
            from django.test.utils import setup_test_environment

            # Setup Django test environment
            setup_test_environment()

            # Create Django test database
            self.old_database_name = settings.DATABASE_NAME
            connection.creation.create_test_db(self.verbosity, autoclobber=True)
        # If Django >= 1.2
        else:
            from django.test.simple import DjangoTestSuiteRunner

            # Initialize Django tests runner
            runner = DjangoTestSuiteRunner(verbosity=self.verbosity)

            # New Django tests runner set ``DEBUG`` to False on setup test
            # environment, so we need to store real ``DEBUG`` value
            DEBUG = settings.DEBUG

            # Setup test environment
            runner.setup_test_environment()

            # And restore it to real value if needed
            if settings.DEBUG != DEBUG:
                settings.DEBUG = DEBUG

            # Setup test databases
            self.old_config = runner.setup_databases()
            self.runner = runner

    def teardown_django(self):
        # If Django < 1.2
        if self.legacy_django:
            from django.db import connection
            from django.test.utils import teardown_test_environment

            # Destroy test database
            connection.creation.destroy_test_db(self.old_database_name,
                                                self.verbosity)

            # Teardown Django test environment
            teardown_test_environment()
        # If Django >= 1.2
        else:
            # Destroy test databases
            self.runner.teardown_databases(self.old_config)

            # Teardown Django test environment
            self.runner.teardown_test_environment()



import re

try:
    from unittest2 import TestCase as BaseTestCase
except ImportError:
    from unittest import TestCase as BaseTestCase

# Dirty hack to prevent ``ImportError`` on installing tddspry via pip
try:
    from nose import tools
except ImportError:
    tools = type('FakeNoseToolsModule', (object, ), {'__all__': []})

from tddspry.utils import *


__all__ = ('NoseTestCase', 'TestCase')


class BaseTestCaseMetaclass(type):

    def __new__(cls, name, bases, attrs):
        for base in bases:
            for base_name in dir(base):
                base_value = getattr(base, base_name)

                if not callable(base_value) or \
                   not base_name.startswith('assert'):
                    continue

                if not base_name in attrs:
                    attrs.update({base_name: base_value})

        for attr_name, attr_value in attrs.items():
            if not attr_name.startswith('assert') or attr_name == 'assert_':
                continue

            if attr_name[6] == '_':
                new_name = underscore_to_camelcase(attr_name)
            else:
                new_name = camelcase_to_underscore(attr_name)

            if not new_name in attrs:
                attrs[new_name] = attr_value

        for attr in tools.__all__:
            attrs.update({attr: getattr(tools, attr)})
            attrs[attr] = staticmethod(attrs[attr])

        return type.__new__(cls, name, bases, attrs)


class TestCaseMetaclass(BaseTestCaseMetaclass):

    def __new__(cls, name, bases, attrs):
        for attr_name, attr_value in attrs.items():
            # Add nose styled names of setup and teardown methods to cls attrs
            if attr_name == 'setup' and not 'setUp' in attrs:
                attrs['setUp'] = attr_value

            if attr_name == 'teardown' and not 'tearDown' in attrs:
                attrs['tearDown'] = attr_value

        return super(TestCaseMetaclass, cls).__new__(cls, name, bases, attrs)


class TestCase(BaseTestCase):
    """
    For convenience this class consist of all functions exists in nose.tools__
    module.

    .. __: http://somethingaboutorange.com/mrl/projects/nose/0.11.0/testing_tools.html

    So, if you have functions, like this::

        def addititon(first, second):
            return first + second

        def division(first, second):
            if not second:
                raise ValueError
            return first / second

        def multiplication(first, second):
            return first * second

        def subtraction(first, second):
            return first - second

    You can write simple ``TestFunctions`` class to test its::

        from tddspry import NoseTestCase


        class TestFunctions(NoseTestCase):

            def test_addition(self):
                self.assert_equal(addititon(1, 2), 3)
                self.assert_not_equal(addititon(1, 2), 2)

            def test_division(self):
                self.assert_equal(division(4, 2), 2)
                self.assert_not_equal(division(5.0, 2), 2)
                self.assert_raises(ValueError, division, 2, 0)

            def test_multiplication(self):
                self.assert_equal(multiplication(1, 2), 2)
                self.assert_not_equal(multiplication(1, 2), 3)

            def test_subtraction(self):
                self.assert_equal(subtraction(2, 1), 1)
                self.assert_equal(subtraction(2, 1), 2)

    """

    __metaclass__ = TestCaseMetaclass

    def assert_not_unicode(self, first, second, message=None):
        """
        Test that ``first`` and ``second`` after converting to ``unicode``
        are not equal.
        """
        return self.assert_not_equal(unicode(first), unicode(second), message)

    def assert_unicode(self, first, second, message=None):
        """
        Test that ``first`` and ``second`` after converting to ``unicode``
        are equal.

        Without ``assert_unicode`` method you may be need to manually convert
        ``first`` and ``second`` values to ``unicode``, when ``assert_unicode``
        make this in place of you.
        """
        return self.assert_equal(unicode(first), unicode(second), message)

    def find_in(self, what, where, flags=None, count=None, escape=False,
                flat=False):
        """
        Try to find ``what`` text in ``where``.

        Method supports both of regular expression matching (with
        ``re.finditer`` function) and ``flat`` string matching (with ``what in
        where`` expression) modes. To enable flat mode call ``find_in`` with
        ``flat=True``.

        If ``what`` string not found, ``AssertionError`` would be raised.

        Specify ``count`` to test that ``what`` occurs ``count`` times in the
        ``where`` text.

        In regular expression mode you can specify ``flags`` supported by
        ``re`` module in text format, like ``'i'`` or ``'s'`` instead of
        ``re.I`` or ``re.S``. If multiple ``flags`` specified they would
        connected with ``|`` (unary OR), e.g. ``flags='iu'`` would be sent to
        ``re.finditer`` as ``flags=re.I | re.U``.

        In reqular expression mode you also can escape ``what`` text with
        ``re.escape`` method. It's useful if your search string contains of
        regular expression metacharacters.

        All non-overlapping matches of ``what`` in ``where`` would be stored
        as list of ``MatchObject`` instances in ``_found`` private var.
        """
        if flat:
            found = None
            real_count = where.count(what)
        else:
            if escape:
                what = re.escape(what)

            regexp = re.compile(what, process_re_flags(flags))
            found = filter(lambda matched: matched, regexp.finditer(where))
            real_count = len(found)

        if found:
            setattr(self, '_found', found)
        elif not found and hasattr(self, '_found'):
            delattr(self, '_found')

        if count is not None:
            assert count == real_count, 'Matched to %r %d times, not %d ' \
                                        'times. Text to search: %r' % \
                                        (what, real_count, count, where)
        else:
            assert real_count, 'No match to %r in %r' % (what, where)

        return True

    def notfind_in(self, what, where, flags=None, escape=False, flat=False):
        """
        Assert that ``what`` not found in ``where``.

        This method is negate version of ``TestCase.find_in`` and supports
        all keywords and modes from it exclude of ``count``. Also in regular
        expression mode ``re.search`` function used instead of ``re.finditer``
        cause we don't need to store matched groups.

        If ``what`` string found, ``AssertionError`` would be raised.
        """
        message = 'Matched %r in %r' % (what, where)

        if not flat:
            if escape:
                what = re.escape(what)

            regexp = re.compile(what, process_re_flags(flags))
            assert not regexp.search(where), message
        else:
            assert not what in where, message

        return True


NoseTestCase = TestCase



import httplib
import re
import warnings

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from urllib import addinfourl

from django.core.management import call_command
from django.core.urlresolvers import NoReverseMatch, reverse
try:
    from django.conf import settings
except ImportError:
    pass
from django.db.models import get_model
from django.utils.encoding import force_unicode
from django.utils.html import escape as real_escape

from tddspry.cases import TestCase as NoseTestCase, \
                          TestCaseMetaclass as NoseTestCaseMetaclass
from tddspry.django import helpers
from tddspry.django.decorators import django_request, show_on_error
from tddspry.django.settings import SITE, DjangoTestCase

from twill import commands
from twill.commands import _parseFindFlags
from twill.errors import TwillAssertionError
from twill.extensions.check_links import check_links
from twill.utils import ResultWrapper


__all__ = ('DatabaseTestCase', 'HttpTestCase', 'TestCase')


class LoginContext(object):

    def __init__(self, testcase, username, password, url=None, formid=None):
        self.testcase = testcase
        formid = formid or 1

        self.testcase.go200(url or settings.LOGIN_URL)

        self.testcase.formvalue(formid, 'username', username)
        self.testcase.formvalue(formid, 'password', password)

        self.testcase.submit200()
        self.testcase.client.login(username=username, password=password)

    def __enter__(self):
        pass

    def __exit__(self, *args):
        self.testcase.logout()


class TestCaseMetaclass(NoseTestCaseMetaclass):

    def __new__(cls, name, bases, attrs):
        for attr_name, attr_value in attrs.items():
            if 'test' in attr_name and callable(attr_value):
                attrs[attr_name] = show_on_error(attr_value, clsname=name)

        # Add twill commands to testcase as staticmethods
        for attr in commands.__all__:
            if attr in ('find', 'go', 'notfind', 'run', 'url'):
                attr_name = '_' + attr
            else:
                attr_name = attr

            attrs.update({attr_name: staticmethod(getattr(commands, attr))})

        attrs.update({'call_command': staticmethod(call_command)})
        attrs.update({'check_links': staticmethod(check_links)})

        # Add ``delete``, ``get``, ``head``, ``options``, ``post``, ``put``
        # methods from Django test client to ``TestCase``
        for attr_name in ('delete', 'get', 'head', 'options', 'post', 'put'):
            attrs.update({attr_name: django_request(attr_name)})
            attrs.update({'%s200' % attr_name: django_request(attr_name, 200)})

        # Dirty hack to convert django testcase camelcase method names to
        # name with underscores
        attr_names = filter(lambda item: item.startswith('assert'),
                            dir(DjangoTestCase))

        for attr_name in attr_names:
            attrs.update({attr_name: getattr(DjangoTestCase, attr_name)})

        super_new = super(TestCaseMetaclass, cls).__new__
        return super_new(cls, name, bases, attrs)


class TestCase(DjangoTestCase, NoseTestCase):

    __metaclass__ = TestCaseMetaclass

    def activate_form(self, formid):
        """
        Set form with ``formid`` as last used by twill. This method useful if
        you got ``TwillException`` with message: ``"more than one form;
        you must select one (use 'fv') before submitting"``.

        Method supports both of numerical and string form ID.
        """
        browser = self.get_browser()
        form = browser.get_form(formid)
        browser._browser.form = form

    def assert_count(self, model_or_manager, mixed, **kwargs):
        """
        Test that number of all ``model_or_manager`` objects equals to given
        ``mixed`` value.

        You can put ``mixed`` argument as ``tuple`` or ``list`` and
        ``assert_count`` checks all of its values.

        Method supports ``using`` keyword, so you can test count objects not
        only in default database.
        """
        manager = self._get_manager(model_or_manager)
        manager, kwargs = self._process_using(manager, kwargs)

        counter = manager.count()

        try:
            manager_from_mixed = self._get_manager(mixed)
            manager_from_mixed, kwargs = \
                self._process_using(manager_from_mixed, kwargs)

            number = manager_from_mixed.count()
        except:
            number = mixed

        if isinstance(number, (list, tuple)):
            equaled = False
            numbers = number

            for number in numbers:
                if number == counter:
                    equaled = True
                    break

            if not equaled:
                assert False, '%r model has %d instance(s), not %s.' % \
                              (manager.model.__name__, counter, numbers)
        else:
            self.assert_equal(counter,
                              number,
                              '%r model has %d instance(s), not %d.' % \
                              (manager.model.__name__, counter, number))

    def assert_contains_count(self, text, count):
        """
        Asserts that ``text`` occuts on current Twill page ``count`` times.

        Else ``TwillAssertionError`` raises.
        """
        return self.find(text, count=count)

    def assert_create(self, model_or_manager, **kwargs):
        """
        Helper tries to create new ``instance`` for ``model_or_manager`` class
        with given ``**kwargs`` and checks that ``instance`` really created.

        Method supports ``using`` keyword, so you can test create process
        not only in default database.

        .. note:: If your model contains ``using`` field, use::

               self.assert_create(model.objects.using('database'),
                                  ...
                                  using='value')

           instead of given ``using`` as keyword argument, cause::

               self.assert_create(model,
                                  ...
                                  using='value')

           will create new ``model`` object in ``default`` database.

        Method returns created ``instance`` if any.
        """
        manager = self._get_manager(model_or_manager)
        manager, kwargs = self._process_using(manager, kwargs)

        old_counter = manager.count()

        instance = manager.create(**kwargs)
        new_counter = manager.count()

        self.assert_equal(new_counter - 1,
                          old_counter,
                          'Could not create only one new %r instance. ' \
                          'New counter is %d, when old counter is %d.' % \
                          (manager.model.__name__, new_counter, old_counter))

        return instance

    def assert_delete(self, mixed, **kwargs):
        """
        Helper tries to delete ``instance`` directly or all objects from
        ``model`` or ``manager`` and checks that its correctly deleted.
        """
        instance, pk = self._get_instance_and_pk(mixed)
        manager = self._get_manager(mixed)
        manager, kwargs = self._process_using(manager, kwargs)
        old_counter = manager.count()

        if pk:
            diff = 1
            instance.delete()
            message = 'Could not delete only one %r instance. New counter is '\
                      '%d, when old counter is %d.'
        else:
            diff = manager.count()
            manager.all().delete()
            message = 'Could not delete all instances of %r model. New ' \
                      'counter is %d, when old counter is %d.'

        new_counter = manager.count()
        message = message % (manager.model.__name__, new_counter, old_counter)

        self.assert_equal(new_counter + diff, old_counter, message)

        if pk:
            try:
                manager.get(pk=pk)
            except manager.model.DoesNotExist:
                pass
            else:
                assert False, 'Could not delete %r instance with %d pk.' % \
                               (manager.model.__name__, pk)

    def assert_not_count(self, model_or_manager, mixed, **kwargs):
        """
        Test that number of all ``model_or_manager`` objects not equals to
        given ``mixed`` value.

        You can put ``mixed`` argument as ``tuple`` or ``list`` and
        ``assert_count`` checks all of its values.

        Method supports ``using`` keyword, so you can test count objects not
        only in default database.
        """
        manager = self._get_manager(model_or_manager)
        manager, kwargs = self._process_using(manager, kwargs)

        counter = manager.count()

        try:
            manager_from_mixed = self._get_manager(mixed)
            manager_from_mixed, kwargs = \
                self._process_using(manager_from_mixed, kwargs)

            number = manager_from_mixed.count()
        except:
            number = mixed

        if isinstance(number, (list, tuple)):
            equaled = False
            numbers = number

            for number in numbers:
                if number == counter:
                    equaled = True
                    break

            if equaled:
                assert False, '%r model has %d instance(s), but should not.' %\
                              (manager.model.__name__, counter)
        else:
            self.assert_not_equal(counter,
                                  number,
                                  '%r model has %d instance(s), but should ' \
                                  'not.' % (manager.model.__name__, number))

    def assert_not_read(self, model_or_manager, query_=None, **kwargs):
        """
        Helper filters ``model_or_manager`` instance by ``query`` or
        ``**kwargs`` lookup and check for empty ``QuerySet``.

        Method support ``using`` keyword so you can test unread models not only
        for default database.
        """
        manager = self._get_manager(model_or_manager)
        manager, kwargs = self._process_using(manager, kwargs)

        if query_:
            queryset = manager.filter(query_)
        else:
            queryset = manager.filter(**kwargs)

        count = queryset.count()

        if count != 0:
            assert False, '%d %s objects exist by %r lookup.' % \
                          (count, manager.model.__name__, query_ or kwargs)

        return True

    def assert_not_unicode(self, first, second, message=None):
        """
        Test that ``first`` and ``second`` after converting with
        ``django.utils.encoding.force_unicode`` function are not equal.
        """
        return self.assert_not_equal(force_unicode(first),
                                     force_unicode(second),
                                     message)

    def assert_read(self, model_or_manager, query_=None, **kwargs):
        """
        Helper filters ``model_or_manager`` instance by ``query`` or
        ``**kwargs`` lookup and checks for valid ``QuerySet``.

        Method returns ``QuerySet`` with filtered instances or simple model
        instance if resulted ``QuerySet`` is only one.

        Method supports ``using`` keyword so you can test read models not only
        from default database.
        """
        manager = self._get_manager(model_or_manager)
        manager, kwargs = self._process_using(manager, kwargs)

        if query_:
            queryset = manager.filter(query_)
        else:
            queryset = manager.filter(**kwargs)

        count = queryset.count()

        if count == 0:
            assert False, 'Could not filter %r objects by %s lookup.' % \
                          (manager.model.__name__, query_ or kwargs)

        if count == 1:
            return queryset[0]

        return queryset

    def assert_unicode(self, first, second, message=None):
        """
        Use ``django.utils.encoding.force_unicode`` to ``first`` and ``second``
        function args.
        """
        return self.assert_equal(force_unicode(first),
                                 force_unicode(second),
                                 message)

    def assert_update(self, mixed, **kwargs):
        """
        Helper tries to update given model instance with ``**kwargs`` and
        checks that all of ``**kwargs`` values was correctly saved to given
        instance. In this case ``assert_update`` returns update instance.

        Also you can update all model class or manager instances with
        QuerySet's ``update`` method and check for these updates.

        Method supports ``using`` keyword so you can test update models not
        only for default database.
        """
        instance, pk = self._get_instance_and_pk(mixed)
        manager = self._get_manager(mixed)
        manager, kwargs = self._process_using(manager, kwargs)

        if pk:
            for name, value in kwargs.items():
                setattr(instance, name, value)
            instance.save()

            instance = manager.get(pk=pk)

            for name, value in kwargs.items():
                self.assert_equal(getattr(instance, name),
                                  value,
                                  'Could not update %r field.' % name)

            return instance

        manager.update(**kwargs)
        return self.assert_read(manager, **kwargs)

    def build_url(self, url, args=None, kwargs=None, prepend=False):
        """
        Helper reverses ``url`` if possible and auto-prepends ``SITE`` to
        it if ``prepend=True``.
        """
        if hasattr(url, 'get_absolute_url'):
            url = url.get_absolute_url()

        if url.startswith(SITE):
            return url

        try:
            url = reverse(url, args=args or [], kwargs=kwargs or {})
        except NoReverseMatch:
            pass

        if not prepend:
            return url

        return SITE + url.lstrip('/')

    def deactivate_form(self):
        """
        Reset last selected form by Twill.
        """
        self.get_browser()._browser.form = None

    def disable_debug(self):
        """
        Disable ``DEBUG`` mode for Django project (set ``settings.DEBUG`` var
        to ``False``).

        To enable use ``TestCase.enable_debug`` method.
        """
        self.enable_debug(False)

    def disable_edit_hidden_fields(self):
        """
        Disable editing hidden fields (``<input type="hidden" ... />``) by
        Twill browser (as by default in twill).

        To enable use ``TestCase.enable_edit_hidden_fields`` method.
        """
        self.enable_edit_hidden_fields(False)

    def disable_redirect(self):
        """
        Disable auto-redirects in Twill tests.

        To enable use ``TestCase.enable_redirect`` method.
        """
        self.enable_redirect(False)

    def enable_debug(self, flag=True):
        """
        Enable ``DEBUG`` mode for Django project (set ``settings.DEBUG`` var
        to ``True``).

        To disable use ``TestCase.disable_debug`` method.
        """
        self.old_DEBUG = settings.DEBUG
        settings.DEBUG = flag

    def enable_edit_hidden_fields(self, flag=True):
        """
        Enable editing hidden fields (``<input type="hidden" ... />``) by
        Twill browser.

        To disable use ``TestCase.disable_edit_hidden_fields`` method.
        """
        self.config('readonly_controls_writeable', flag)

    def enable_redirect(self, flag=True):
        """
        Enable auto-redirects in Twill tests (as by default in twill).

        To disable use ``TestCase.disable_redirect`` method.
        """
        self.config('acknowledge_equiv_refresh', int(flag))

    def find(self, what, flags='', flat=False, count=None, escape=False):
        """
        Twill used regexp for searching content on web-page. Use ``flat=True``
        to search content on web-page by standart Python ``what in html``
        expression.

        If this expression was not True (was not found on page) it's raises
        ``TwillAssertionError`` as in ``twill.commands.find`` method.

        Specify ``count`` to test that ``what`` occurs ``count`` times in the
        content of the web-page.

        You could escape ``what`` text by standart ``django.utils.html.escape``
        function if call method with ``escape=True``, like::

            self.find('Text with "quotes"', escape=True)

        """
        if escape:
            what = real_escape(what)

        if not flat and not count:
            return self._find(what, flags)

        html = self.get_browser().get_html()
        real_count = html.count(what)

        if count is not None and count != real_count:
            raise TwillAssertionError('Matched to %r %d times, not %d ' \
                                      'times.' % (what, real_count, count))
        elif real_count == 0:
            raise TwillAssertionError('No match to %r' % what)

        return True

    def find_in(self, what, where, flags='', flat=False, count=None,
                escape=False):
        """
        Alternate version of ``find`` method that allow to find text in another
        text, not only in current loaded page.

        ``find_in`` supports all keywords from original ``find`` method.

        If ``what`` not found in ``where``, ``find_in`` raises
        ``TwillAssertionError``.
        """
        if escape:
            what = real_escape(what)

        if not flat and not count:
            regexp = re.compile(what, _parseFindFlags(flags))

            if not regexp.search(where):
                self.text_to_twill(where)
                raise TwillAssertionError('No match to %r' % what)

            return True

        real_count = where.count(what)

        if count is not None and count != real_count:
            self.text_to_twill(where)
            raise TwillAssertionError('Matched to %r %d times, not %d ' \
                                      'times.' % (what, real_count, count))
        elif real_count == 0:
            self.text_to_twill(where)
            raise TwillAssertionError('No match to %r' % what)

        return True

    def find_url(self, url, args=None, kwargs=None, prepend=False, flags='',
                 flat=False, count=None, escape=False):
        """
        Helper method to build url and find it on current web-page.

        ::

            self.find_url('index')

        equals to::

            self.find(self.build_url('index'))

        You should use all of ``build_url`` and ``find`` keyword arguments,
        like ``args`` for ``build_url`` or ``count`` for ``find``.
        """
        return self.find(self.build_url(url, args, kwargs, prepend),
                         flags=flags,
                         flat=flat,
                         count=count,
                         escape=escape)

    def fix_xhtml(self):
        """
        If Twill fails when there is xhtml output, call these method before
        first Twill function, say in ``TestCase.setup``.

        Alternate, you should enable ``xhtml`` attribute of your inherited
        ``TestCase``, e.g.::

            class TestHttp(TestCase):

                xhtml = True

                ...

        This also enables html mode for default Twill browser.
        """
        self.get_browser()._browser._factory.is_html = True

    def follow200(self, what, url=None, args=None, kwargs=None,
                  check_links=False):
        """
        Find the first matching link on the page, visit it and check that
        response code is 200.

        If ``url`` is set function checks to make sure that the current URL
        matches the given regexp or urlpattern.

        If ``check_links`` is set function checks links on current page.
        """
        self.follow(what)
        self.code(200)

        if url:
            self.url(url, args, kwargs)

        if check_links:
            self.check_links()

    def go(self, url, args=None, kwargs=None):
        """
        Twill needs to set full URL of web-page to loading. This helper
        auto-prepends ``SITE`` value if needed.

        You can also give urlpattern name and function tries to ``reverse``
        it to real URL.
        """
        url = self.build_url(url, args, kwargs, True)
        return self._go(url)

    def go200(self, url, args=None, kwargs=None, check_links=False):
        """
        Go to url and check that response code is 200.
        """
        self.go(url, args, kwargs)
        self.code(200)

        if check_links:
            self.check_links()

    def helper(self, name, *args, **kwargs):
        return getattr(helpers, name)(self, *args, **kwargs)

    def _get_helpers(self):
        return helpers

    helpers = property(_get_helpers)

    def login(self, username, password, url=None, formid=None):
        """
        Login to Django using ``username`` and ``password``.

        Also login to Django test client.
        """
        return LoginContext(self, username, password, url, formid)

    def login_to_admin(self, username, password):
        """
        Login to Django admin CRUD using ``username`` and ``password``.

        Also login to Django test client.
        """
        self.go200('/admin/')

        self.formvalue(1, 'username', username)
        self.formvalue(1, 'password', password)

        self.submit200()
        self.notfind('<input type="hidden" name="this_is_the_login_form" ' \
                     'value="1" />')

        self.client.login(username=username, password=password)

    def logout(self, url=None):
        """
        Logout from current Django session.

        Also logout from Django test client.
        """
        self.go200(url or settings.LOGOUT_URL)
        self.client.logout()

    def notfind(self, what, flags='', flat=False, escape=False):
        """
        Twill used regexp for searching content on web-page. Use ``flat=True``
        to search content on web-page by standart Python ``not what in html``
        expression.

        If this expression was not True (was found on page) it's raises
        ``TwillAssertionError`` as in ``twill.commands.notfind`` method.

        You could escape ``what`` text by standart ``django.utils.html.escape``
        function if call method with ``escape=True``, like::

            self.notfind('Text with "quotes"', escape=True)

        """
        if escape:
            what = real_escape(what)

        if not flat:
            return self._notfind(what, flags)

        html = self.get_browser().get_html()
        if what in html:
            raise TwillAssertionError('Match to %r' % what)

        return True

    def notfind_in(self, what, where, flags='', flat=False, escape=False):
        """
        Alternate version of ``notfind`` method that allow to check that text
        not found in another text, not only in current loaded page.

        ``notfind_in`` supports all keywords from original ``notfind`` method.

        If ``what`` found in ``where``, ``notfind_in`` raises
        ``TwillAssertionError``.
        """
        found = False

        if escape:
            what = real_escape(what)

        if flat and what in where:
            found = True
        elif not flat:
            regexp = re.compile(what, _parseFindFlags(flags))
            if regexp.search(where):
                found = True

        if found:
            self.text_to_twill(where)
            raise TwillAssertionError('Match to %r' % what)

        return True

    def notfind_url(self, url, args=None, kwargs=None, prepend=False, flags='',
                    flat=False, escape=False):
        """
        Helper method to build url and not find it on current web-page.

        ::

            self.notfind_url('index')

        equals to::

            self.notfind(self.build_url('index'))

        You should use all of ``build_url`` and ``notfind`` keyword arguments,
        like ``args`` for ``build_url`` or ``flat`` for ``notfind``.
        """
        return self.notfind(self.build_url(url, args, kwargs, prepend),
                            flags=flags,
                            flat=flat,
                            escape=escape)

    def response_to_twill(self, response):
        """
        Wrap Django response to work with Twill.
        """
        path = response.request.get('PATH_INFO')
        url = path and SITE + path.lstrip('/') or path

        headers_msg = '\n'.join('%s: %s' % (k, v) for k, v in response.items())
        headers_msg = StringIO(headers_msg)
        headers = httplib.HTTPMessage(headers_msg)

        io_response = StringIO(response.content)
        urllib_response = addinfourl(io_response,
                                     headers,
                                     url,
                                     response.status_code)
        urllib_response._headers = headers
        urllib_response._url = url
        urllib_response.msg = u'OK'
        urllib_response.seek = urllib_response.fp.seek

        self.get_browser()._browser._set_response(urllib_response, False)
        self.get_browser().result = ResultWrapper(response.status_code,
                                                  url,
                                                  response.content)

        self._apply_xhtml()

    def submit200(self, submit_button=None, url=None, check_links=False):
        """
        Submit form and checks that response code is 200.
        """
        self.submit(submit_button)
        self.code(200)

        if url is not None:
            self.url(url)

        if check_links:
            self.check_links()

    def text_to_twill(self, text):
        """
        Wrap text to work with Twill.
        """
        headers_msg = 'Content: text-plain; encoding=utf-8\n'
        headers_msg = StringIO(headers_msg)
        headers = httplib.HTTPMessage(headers_msg)

        status_code = 200
        url = 'text://'

        io_response = StringIO(text)
        urllib_response = addinfourl(io_response,
                                     headers,
                                     url,
                                     status_code)
        urllib_response._headers = headers
        urllib_response._url = url
        urllib_response.msg = u'OK'
        urllib_response.seek = urllib_response.fp.seek

        self.get_browser()._browser._factory.set_response(urllib_response)
        self.get_browser().result = ResultWrapper(status_code, url, text)

        self._apply_xhtml()

    def url(self, url, args=None, kwargs=None, regexp=True):
        """
        Assert that current URL matches the given regexp.

        If ``regexp`` is set function appends '$' to the regexp end if needed.
        """
        should_be = self.build_url(url, args, kwargs, True)

        if regexp and should_be[-1] != '$':
            should_be += '$'

        return self._url(should_be)

    def _apply_disabled_apps(self):
        """
        Remove apps from ``disabled_apps`` attribute and/or from
        ``TDDSPRY_DISABLED_APPS`` settings var.
        """
        disabled_apps = getattr(settings, 'TDDSPRY_DISABLED_APPS', None)

        # Backward compatible version
        if disabled_apps is None:
            disabled_apps = getattr(settings, 'TEST_DISABLED_APPS', [])

        if not hasattr(self, 'disabled_apps'):
            self.disabled_apps = disabled_apps
        else:
            self.disabled_apps = list(self.disabled_apps)
            self.disabled_apps.extend(disabled_apps)

        if not self.disabled_apps:
            return

        self.old_INSTALLED_APPS = settings.INSTALLED_APPS
        settings.INSTALLED_APPS = []

        for app in self.old_INSTALLED_APPS:
            if not app in self.disabled_apps:
                settings.INSTALLED_APPS.append(app)

    def _apply_xhtml(self):
        """
        Apply html mode for default Twill browser only if needed.
        """
        if hasattr(self, 'xhtml') and self.xhtml:
            self.fix_xhtml()

    def _get_instance_and_pk(self, mixed):
        """
        Utility function to return tuple contains of models instance and its pk
        if possible.
        """
        instance, pk = None, None

        if hasattr(mixed, 'pk') and not isinstance(mixed.pk, property):
            instance, pk = mixed, mixed.pk

        return (instance, pk)

    def _get_manager(self, model_or_manager):
        """
        Utility function to return default manager from model or instance
        object or given manager.
        """
        if isinstance(model_or_manager, basestring):
            app, model = model_or_manager.split('.')
            model_or_manager = get_model(app, model)

        if hasattr(model_or_manager, '_default_manager'):
            return model_or_manager._default_manager

        return model_or_manager

    def _post_teardown(self):
        """
        Renew ``settings.DEBUG`` value if needed.
        """
        super(TestCase, self)._post_teardown()

        if hasattr(self, 'old_DEBUG'):
            settings.DEBUG = self.old_DEBUG

        if hasattr(self, 'old_INSTALLED_APPS'):
            settings.INSTALLED_APPS = self.old_INSTALLED_APPS

    def _pre_setup(self):
        """
        Remove disabled apps from project installed and enable html mode for
        default Twill browser if needed.
        """
        self._apply_disabled_apps()
        super(TestCase, self)._pre_setup()
        self._apply_xhtml()

    def _process_using(self, manager, kwargs):
        """
        Utility function to check if model of manager has ``using`` field and
        if not use it as arg for ``manager.using`` function.
        """
        if 'using' in kwargs:
            model = manager.model

            all_fields = model._meta.fields + model._meta.many_to_many
            found = False

            for field in all_fields:
                if field.name == 'using':
                    found = True
                    break

            if not found:
                using = kwargs.pop('using')
                return (manager.using(using), kwargs)

        return (manager, kwargs)


class DatabaseTestCase(TestCase):

    def __init__(self, *args, **kwargs):
        super(DatabaseTestCase, self).__init__(*args, **kwargs)
        self._warning_message = \
            'Calling super for ``%s()`` method is deprecated. ' \
            '``tddspry.django.TestCase`` class does not need this anymore.'

    def setup(self):
        warnings.warn(self._warning_message % 'setup', DeprecationWarning)

    def teardown(self):
        warnings.warn(self._warning_message % 'teardown', DeprecationWarning)


HttpTestCase = DatabaseTestCase



import re


__all__ = ('camelcase_to_underscore', 'process_re_flags',
           'underscore_to_camelcase')


RE_FLAGS = {'i': re.I, 'l': re.L, 'm': re.M, 's': re.S, 'u': re.U, 'x': re.X}


def camelcase_to_underscore(name):
    """
    Convert old-style camel-case method names to new-style names with
    underscores.

    ::

        >>> camelcase_to_underscore('assertEqual')
        ... 'assert_equal'
        >>> camelcase_to_underscore('assertNotEqual')
        ... 'assert_not_equal'
        >>> camelcase_to_underscore('assert_equal')
        ... 'assert_equal'
        >>> camelcase_to_underscore('assert_not_equal')
        ... 'assert_not_equal'

    """
    new_name = u''

    for char in name:
        if char.isupper():
            char = char.lower()
            new_name += u'_'

        new_name += char

    return new_name


def process_re_flags(flags=None):
    """
    Specify ``flags`` supported by ``re`` module in text format, like ``'i'``
    or ``'s'`` instead of ``re.I`` or ``re.S``. If multiple ``flags`` specified
    they would connected with ``|`` (unary OR), e.g. ``'iu'`` would be
    processed as ``re.I | re.U``.

    All supported flags are: ``'i', 'l', 'm', 's', 'u', 'x'``.
    """
    if flags:
        old_flags = flags
        flags = None

        for flag in old_flags:
            flag = flag.lower()

            if not flag in RE_FLAGS:
                continue

            flag = RE_FLAGS[flag]

            if flags is None:
                flags = flag
            else:
                flags |= flag

    return flags or 0


def underscore_to_camelcase(name):
    """
    Convert new-style method names with underscores to old style camel-case
    names.

    ::

        >>> underscore_to_camelcase('assert_equal')
        ... 'assertEqual'
        >>> underscore_to_camelcase('assert_not_equal')
        ... 'assertNotEqual'
        >>> underscore_to_camelcase('assertEqual')
        ... 'assertEqual'
        >>> underscore_to_camelcase('assertNotEqual')
        ... 'assertNotEqual'

    """
    return name[0].lower() + \
           name.replace('_', ' ').title().replace(' ', '')[1:]



"""
``tddspry.django`` provides ``DatabaseTestCase`` and ``HttpTestCase`` classes
to testing Django applications.

Database tests with ``DatabaseTestCase``
----------------------------------------

For testing Django model classes and anything objects that needed database
connection you should be used ``tddspry.django.DatabaseTestCase``. This
test-case inherits ``tddspry.TestCase``, so consists of all nose tools
functions and decorators.

At ``setup`` ``DatabaseTestCase`` creates test database or flushes it if
needed and loading fixtures if possible (``database_name``,
``database_flush`` and ``fixtures`` attributes).

Also for convenience ``DatabaseTestCase`` has additional helpers:

.. autoclass :: tddspry.django.DatabaseTestCase
   :members: assert_count, assert_create, assert_delete, assert_read,
             assert_update

**Note:** Also ``DatabaseTestCase`` can call additional helper by
``helper`` method. `See below`_ how use it.

Usage
~~~~~

So, for testing CRUD_ of ``django.contrib.auth.models.Group`` model you can
write next test case::

    from tddspry.django import DatabaseTestCase

    from django.contrib.auth.models import Group


    NEW_NAME = 'Super-Test Group'
    TEST_NAME = 'Test Group'


    class TestGroup(DatabaseTestCase):

        def test_create(self):
            self.assert_create(Group, name=TEST_NAME)

        def test_delete(self):
            group = self.assert_create(Group, name=TEST_NAME)
            self.assert_delete(group)

        def test_read(self):
            self.assert_create(Group, name=TEST_NAME)
            self.assert_read(Group, name=TEST_NAME)

        def test_update(self):
            group = self.assert_create(Group, name=TEST_NAME)
            self.assert_update(group, name=NEW_NAME)

.. _`See below`: `Additional helpers`
.. _CRUD: http://en.wikipedia.org/wiki/Create,_read,_update_and_delete

Server-side tests with ``HttpTestCase``
---------------------------------------

In addition to :class:`tddspry.django.DatabaseTestCase` :mod:`tddspry.django`
provides :class:`tddspry.django.HttpTestCase` class to testing HTTP responses
with `Twill browser`_ and `django.test.Client`_.

At setup ``HttpTestCase`` run Django WSGI server and connects it with
twill browser.

For historical reasons ``HttpTestCase`` was developed for testing Django
applications with Twill browser, so its consists of all functions exists
in `twill.commands`_ module as class methods.

And for convenience several twill methods was simplifying and rewriting, there
are:

.. autoclass :: tddspry.django.HttpTestCase
   :members: find, go, notfind, url

And from :ref:`release_0.3` of ``tddspry`` ``HttpTestCase`` consist of ``client``
attribute that stores instance of ``django.test.Client`` class.

Also, ``HttpTestCase`` provides next methods:

.. autoclass :: tddspry.django.HttpTestCase
   :members: build_url, disable_edit_hidden_fields, disable_redirect,
             enable_edit_hidden_fields, enable_redirect, go200, login,
             login_to_admin, logout, submit200

Usage
~~~~~

So, for testing login/logout process in your project you can write next
test-case::

    from tddspry.django import HttpTestCase

    from django.conf import settings


    class TestLoginPage(HttpTestCase):

        def test_login(self):
            user = self.helper('create_user', 'username', 'password')
            self.login('username', 'password')
            self.url(settings.LOGIN_REDIRECT_URL)

        def test_logout(self):
            user = self.helper('create_user', 'username', 'password')
            self.login('username', 'password')
            self.logout()
            self.url('/')

.. _`Twill browser`: http://twill.idyll.org/
.. _`django.test.Client`: http://docs.djangoproject.com/en/dev/topics/testing/#module-django.test.client
.. _`twill.commands`: http://twill.idyll.org/commands.html

"""

from tddspry.django.cases import *



#!/usr/bin/env python
"""
Custom ``nosetests`` runner that make possible to run ``nosetests`` for your
Django projects without installed ``setuptools``.
"""

import os
import sys

import nose
from nose.plugins.manager import EntryPointPluginManager

from tddspry.noseplugins import DjangoPlugin


def main():
    # First of all add current work directory to ``sys.path`` if it not there
    cwd = os.getcwd()

    if not cwd in sys.path or not cwd.strip(os.sep) in sys.path:
        sys.path.append(cwd)

    # Try to find that DjangoPlugin loaded from entry_points or not
    found, kwargs = False, {}

    manager = EntryPointPluginManager()
    manager.loadPlugins()

    for plugin in manager.plugins:
        if isinstance(plugin, DjangoPlugin):
            found = True
            break

    # If not manually add
    if not found:
        kwargs = {'addplugins': [DjangoPlugin()]}

    # Enable DjangoPlugin
    os.environ['NOSE_WITH_DJANGO'] = '1'

    # Run ``nosetests``
    nose.main(**kwargs)


if __name__ == '__main__':
    main()



import re

from django.core import mail
from django.contrib.auth.models import User

from tddspry.django.helpers import EMAIL, PASSWORD, USERNAME


__all__ = ('registration', )


def activate(obj, word='activate', url='registration_activate'):
    """
    For registered user get activation code from mailbox.
    """
    obj.assert_equal(len(mail.outbox), 1)

    body = mail.outbox[0].body
    match = re.search('.*%s/(.*)' % word, body)

    assert match, \
          'Failed to find proper activation link in the mail: %s' % body

    code = match.groups()[0]
    obj.go200(url, args=[code[:-1]])


def registration(obj, username=None, email=None, password=None,
                 verbosity=False,
                 registration_url='registration_register',
                 registration_formid=1,
                 registration_tos=False,
                 activate_word='activate',
                 activation_url='registration_activate',
                 login_url=None,
                 login_formid=1):
    """
    Register new user, activate it and login. Registers new user, activates it
    and tries to log in. Helper covers all registration process provided by
    `django-registration`_.

    .. _`django-registration`: http://code.google.com/p/django-registration/

    """
    # Go to registration page
    obj.go200(registration_url)
    obj.find('<input id="id_username" type="text" class="required" ' \
             'name="username"')

    submit(obj, username, email, password, registration_formid,
           registration_tos)

    if verbosity:
        obj.show()

    # Get created user
    user = User.objects.get(username=USERNAME)

    # Created user should not be active
    obj.assert_false(user.is_active,
                     'User is already activated and should not.')

    activate(obj, activate_word, activation_url)

    if verbosity:
        obj.show()

    # Test user exists and is_active
    user = User.objects.get(username=USERNAME)
    obj.assert_true(user.is_active, 'User is not activated yet.')

    # Test login
    obj.login(USERNAME, PASSWORD, login_url, login_formid)


def submit(obj, username=None, email=None, password=None, formid=None,
           tos=False):
    """
    Submit form for register new user. Note - if you have captcha
    in the form, you need to mock it.
    """
    formid = formid or 1

    obj.formvalue(formid, 'username', username or USERNAME)
    obj.formvalue(formid, 'email', email or EMAIL)
    obj.formvalue(formid, 'password1', password or PASSWORD)
    obj.formvalue(formid, 'password2', password or PASSWORD)

    if tos:
        obj.formvalue(formid, 'tos', 'on')

    obj.submit200()



from django.conf import settings
from django.test import TestCase

from nose.util import resolve_name


__all__ = ('IP', 'PORT', 'SITE', 'DjangoTestCase')


IP = getattr(settings, 'TDDSPRY_IP', '127.0.0.1')
PORT = getattr(settings, 'TDDSPRY_PORT', 8088)
SITE = 'http://%s:%s/' % (IP, PORT)

DjangoTestCase = getattr(settings, 'TDDSPRY_TEST_CASE', TestCase)

if isinstance(DjangoTestCase, basestring):
    DjangoTestCase = resolve_name(DjangoTestCase)



import os
import time

from functools import wraps

from twill.commands import browser, save_html, show, showforms
from twill.errors import TwillAssertionError, TwillException
from twill.utils import print_form


__all__ = ('django_request', 'show_on_error')


def django_request(name, code=None):
    """
    Makes a GET, POST, HEAD, OPTIONS, PUT or DELETE request (reads from
    ``name``) with Django test client on the provided ``url`` and returns a
    Response object, which is wrapped to work with Twill.

    ``url`` passed to this method auto-builds with ``build_url`` method,
    so you can to provide ``args`` and ``kwargs`` args to reverse
    urlpattern name in ``url``.

    Decorator returns Django response.
    """

    def decorator(obj, url, args=None, kwargs=None, data=None, follow=False,
                  **extra):
        # Make able to use Django syntax for requests with data
        if isinstance(args, dict) and not data:
            data = args
            args = None

        check_links = extra.pop('check_links', False)

        # Build URL
        url = obj.build_url(url, args, kwargs)

        # Make request
        data = data or {}
        request = getattr(obj.client, name)
        response = request(url, data, follow=follow, **extra)

        # Wrap Django response to work with Twill.
        obj.response_to_twill(response)

        # Show some debug info
        print('%s ==> %s') % (name.upper(), browser.get_url())

        # Check for code if needed
        if code:
            obj.code(code)

        # Check for links if needed
        if check_links:
            obj.check_links()

        return response

    # Add fancy docstring to resulted method
    allnames = 'GET, POST, HEAD, OPTIONS, PUT or DELETE'
    deletes = (' (reads from', '``name``) ')

    decorator.__doc__ = django_request.__doc__.replace(allnames, name.upper())
    decorator.__doc__ = decorator.__doc__.replace('Decorator', 'Method')

    for delete in deletes:
        decorator.__doc__ = decorator.__doc__.replace(delete, '')

    decorator.__name__ = code and '%s%d' % (name, code) or name

    return decorator


def show_on_error(func, clsname=None):
    """
    On ``TwillAssertionError``, show last page got by twill.

    On ``TwillException: no field matches``, call twill ``showforms`` command.

    By default, decorator show output in stdout. To store output somewhere in
    disk - set up ``TWILL_ERROR_DIR`` environment var, e.g. if some error
    cathced in tests ran by::

        TWILL_ERROR_DIR=/tmp/tddspry/ django-nosetests.py

    all output would be stored in ``/tmp/tddspry/`` dir.
    """

    def save_forms(filename):
        forms = browser.get_all_forms()
        stream = open(filename, 'w')

        for i, form in enumerate(forms):
            print_form(i, form, stream)

        stream.close()

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except (TwillAssertionError, TwillException), e:
            field_error = False

            # Ignore all ``TwillException`` errors, but "no field matches"
            if e.__class__ == TwillException:
                message = e.args[0]

                if not message.startswith('no field matches "'):
                    raise

                field_error = True

            cmd = field_error and showforms or show
            saved, showed = False, False

            if 'TWILL_ERROR_DIR' in os.environ:
                dirname = os.environ['TWILL_ERROR_DIR']

                if not os.path.isdir(dirname):
                    try:
                        os.mkdir(dirname)
                    except:
                        print('Cannot create directory at %r.') % dirname
                        cmd()

                        showed = True

                if not showed:
                    timestamp = int(time.time())

                    if clsname:
                        filename = '%s.%s.%s' % (func.__module__,
                                                    clsname,
                                                    func.__name__)
                    else:
                        filename = '%s.%s' % (func.__module__,
                                                func.__name__)

                    format = field_error and 'txt' or 'html'
                    filename = '%s-%d.%s' % (filename, timestamp, format)
                    filename = os.path.abspath(os.path.join(dirname, filename))

                    try:
                        if field_error:
                            save_forms(filename)
                        else:
                            save_html(filename)
                    except:
                        print('Cannot write to file at %r.') % filename
                        cmd()

                        showed = True
                    else:
                        print('Output saved to %r.') % filename
                        saved = True

            if not saved and not showed:
                cmd()

            raise

    return wrapper



#!/usr/bin/env python
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
    kwargs = {}
else:
    kwargs = {
        'entry_points': """
        [nose.plugins.0.10]
        django = tddspry.noseplugins:DjangoPlugin
        """,
    }


readme = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
README = readme.read()
readme.close()

requires = ['Django (>= 1.0)', 'nose (>= 0.11.0)', 'twill (>= 0.9)']
version = __import__('tddspry').get_version()

if kwargs:
    install_requires = [r.replace('(', '').replace(')', '') for r in requires \
                        if not r.startswith('Django')]
    kwargs.update({'install_requires': install_requires,
                   'setup_requires': install_requires})

setup(name='tddspry',
      version=version,
      description='Collection of test cases and additional helpers to test ' \
                  'Django applications with nose library.',
      long_description=README,

      author='Igor Davydenko',
      author_email='playpauseandstop@gmail.com',
      url='http://github.com/playpauseandstop/tddspry',

      packages=['tddspry', 'tddspry.django', 'tddspry.django.helpers',
                'tddspry.noseplugins'],
      scripts=['bin/django-nosetests.py'],

      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Django',
          'License :: OSI Approved :: BSD License',
          'Topic :: Software Development :: Testing',
      ],
      keywords='django nose tdd testing tests twill',

      requires=requires,

      **kwargs)
